﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WpfApplication7
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		ViewModel vm;
		public MainWindow()
		{
			InitializeComponent();
			this.Loaded += MainWindow_Loaded;
			this.SizeChanged += MainWindow_SizeChanged;
		}

		void MainWindow_Loaded(object sender, RoutedEventArgs e)
		{
			this.vm = new ViewModel((int)(this.ActualHeight / 130) * (int)(this.ActualWidth / 130));
			this.DataContext = this.vm;
		}

		void MainWindow_SizeChanged(object sender, SizeChangedEventArgs e)
		{
			if (this.vm != null)
				this.vm.LoadSize = (int)(e.NewSize.Height / 130) * (int)(e.NewSize.Width / 130);
		}

		private void LoadNext(object sender, RoutedEventArgs e)
		{
			vm.LoadNext();
		}

		private void LoadPrevious(object sender, RoutedEventArgs e)
		{
			vm.LoadPrevious();
		}
	}

	public class ViewModel : INotifyPropertyChanged
	{
		public ViewModel(int loadSize)
		{
			this.loadSize = loadSize;
			VQCV.ItemsLoaded += VQCV_ItemsLoaded;
		}

		private readonly IEnumerable<Item> items = Enumerable.Range(0, 10000).Select(i => new Item("Item" + i));

		private IEnumerable<Item> current;
		public IEnumerable<Item> CurrentPage
		{
			get { return current; }
			set
			{
				current = value;
				this.OnPropertyChanged("CurrentPage");
			}
		}

		private IEnumerable<Item> previous;
		public IEnumerable<Item> PreviousPage
		{
			get { return previous; }
			set
			{
				previous = value;
				this.OnPropertyChanged("PreviousPage");
			}
		}

		private IEnumerable<Item> next;
		public IEnumerable<Item> NextPage
		{
			get { return next; }
			set
			{
				next = value;
				this.OnPropertyChanged("NextPage");
			}
		}

		private int loadSize;

		public int LoadSize
		{
			get
			{
				return loadSize;
			}
			set
			{
				if (loadSize != value)
				{
					this.VQCV.LoadSize = value;
					loadSize = value;
					this.Load();
				}
			}
		}

		private void Load()
		{
			if (this.lastOperationPrevious)
			{
				this.lastOperationPrevious = false;
				this.CurrentIndex += loadSize;
			}
			if (this.lastOperationNext)
			{
				this.lastOperationNext = false;
				this.CurrentIndex -= loadSize;
			}

			var items = Enumerable.Range(0, loadSize).Select(i => VQCV[CurrentIndex + i]).OfType<Item>().ToList();
			if (items.Count > 0)
			{
				this.CurrentPage = items;
			}
			else
			{
				this.currentFlag = true;
			}

			items = Enumerable.Range(0, loadSize).Select(i => VQCV[Math.Max(CurrentIndex - loadSize, 0) + i]).OfType<Item>().ToList();
			if (items.Count > 0)
			{
				this.PreviousPage = items;
			}
			else
			{
				this.previousFlag = true;
			}

			items = Enumerable.Range(0, loadSize).Select(i => VQCV[CurrentIndex + loadSize + i]).OfType<Item>().ToList();
			if (items.Count > 0)
			{
				this.NextPage = items;
			}
			else
			{
				this.nextFlag = true;
			}

		}

		bool currentFlag = false;
		bool nextFlag = false;
		bool previousFlag = false;

		bool lastOperationNext = false;
		bool lastOperationPrevious = false;

		void VQCV_ItemsLoaded(object sender, Telerik.Windows.Data.VirtualQueryableCollectionViewItemsLoadedEventArgs e)
		{
			if (this.nextFlag)
			{
				this.NextPage = e.Items.OfType<Item>();
				this.nextFlag = false;
			}
			if (this.previousFlag)
			{
				this.PreviousPage = e.Items.OfType<Item>();
				this.previousFlag = false;
			}
			if (this.currentFlag)
			{
				this.CurrentPage = e.Items.OfType<Item>();
				this.currentFlag = false;
			}
		}

		public int CurrentIndex { get; set; }

		public void LoadPrevious()
		{
			this.NextPage = this.CurrentPage;
			this.CurrentPage = this.PreviousPage;

			this.CurrentIndex -= lastOperationNext ? loadSize * 3 : loadSize;
			lastOperationNext = false;
			lastOperationPrevious = true;

			var items = Enumerable.Range(0, loadSize).Select(i => VQCV[CurrentIndex + i]).OfType<Item>().ToList();

			if (items.Count > 0)
			{
				this.PreviousPage = items;
			}
			else
			{
				this.previousFlag = true;
			}
		}

		public void LoadNext()
		{
			this.PreviousPage = this.CurrentPage;
			this.CurrentPage = this.NextPage;

			this.CurrentIndex += lastOperationPrevious ? loadSize * 3 : loadSize;
			lastOperationNext = true;
			lastOperationPrevious = false;

			this.nextFlag = true;
			var items = Enumerable.Range(0, loadSize).Select(i => VQCV[CurrentIndex + i]).OfType<Item>().ToList();
			if (items.Count > 0)
			{
				this.NextPage = items;
			}
			else
			{
				this.nextFlag = true;
			}
		}

		private Telerik.Windows.Data.VirtualQueryableCollectionView vqcv;

		public Telerik.Windows.Data.VirtualQueryableCollectionView VQCV
		{
			get
			{
				if (vqcv == null)
				{
					vqcv = new Telerik.Windows.Data.VirtualQueryableCollectionView(items);
					vqcv.VirtualItemCount = 10000;
					vqcv.LoadSize = this.LoadSize;
				}
				return vqcv;
			}
		}

		public void OnPropertyChanged(string propertyName)
		{
			if (this.PropertyChanged != null)
			{
				this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}

		public event PropertyChangedEventHandler PropertyChanged;
	}

	public class Item
	{
		public Item(string name)
		{
			Name = name;
		}

		public string Name { get; set; }
	}
}
